import { ComponentType, ReactElement } from "react";
import { Route, Routes } from "react-router-dom";
import { ErrorBoundary } from "../errorBoundary";
import { DefaultTitleContextProvider } from "../title";
import { CoreAdminRoutes, CoreAdminRoutesProps } from "./CoreAdminRoutes";
import { InAdminContext, useInAdminContext } from "./InAdminContext";
import { useInAppContext } from "./InAppContext";

export interface CoreAdminContextProps extends CoreAdminRoutesProps {
  /**
   * The base path for all URLs generated by react-admin.
   *
   * @see https://marmelab.com/react-admin/Admin.html#using-react-admin-in-a-sub-path
   * @example
   * import { Admin } from 'react-admin';
   * import { BrowserRouter } from 'react-router-dom';
   * import { dataProvider } from './dataProvider';
   *
   * const App = () => (
   *     <BrowserRouter>
   *         <Admin basename="/admin" dataProvider={dataProvider}>
   *              ...
   *         </Admin>
   *    </BrowserRouter>
   * );
   */
  basepath?: string;

  /**
   * The component displayed when an error is caught in a child component
   * @see https://marmelab.com/react-admin/Admin.html#error
   * @example
   * import { Admin } from 'react-admin';
   * import { MyError } from './error';
   *
   * const App = () => (
   *     <Admin error={MyError}>
   *         ...
   *     </Admin>
   * );
   */
  error?: ComponentType<any> | ReactElement | null;

  /**
   * The title of the error page
   * @see https://marmelab.com/react-admin/Admin.html#title
   * @example
   * import { Admin } from 'react-admin';
   * import { dataProvider } from './dataProvider';
   *
   * const App = () => (
   *    <Admin title="My Admin" dataProvider={dataProvider}>
   *       ...
   *   </Admin>
   * );
   */
  title?: string | ReactElement | null;
}

export function CoreAdminContext(props: CoreAdminContextProps) {
  const {
    catchAll,
    children,
    dashboard,
    error,
    layout,
    loading,
    ready,
    requireAuth,
    title,
  } = props;

  const inApp = useInAppContext();
  const inAdmin = useInAdminContext();

  if (!inApp) {
    throw new Error(
      "You've tried to access admin context outside <AppContext />.\n" +
      "Please wrap your code with it first.\n"
    );
  }

  if (inAdmin) {
    throw new Error(
      "You've tried to access admin context inside another <AdminContext />.\n" +
      "Please unwrap your code with it first.\n"
    );
  }

  return (
    <DefaultTitleContextProvider value={title}>
      <ErrorBoundary error={error}>
        <InAdminContext>
          <Routes>
            <Route
              path="/*"
              element={
                <CoreAdminRoutes
                  catchAll={catchAll}
                  dashboard={dashboard}
                  layout={layout}
                  loading={loading}
                  requireAuth={requireAuth}
                  ready={ready}
                >
                  {children}
                </CoreAdminRoutes>
              }
            />
          </Routes>
        </InAdminContext>
      </ErrorBoundary>
    </DefaultTitleContextProvider>
  );
}
