import { ComponentType, ReactElement, useMemo } from "react";
import { Route as ReactRoute, Routes } from "react-router-dom";
import { CoreAppRoutes, CoreAppRoutesProps } from "./CoreAppRoutes";
import { defaultStore, Store, StoreContextProvider } from "../store";
import { AuthContext, AuthProvider } from "../auth";
import { I18nContextProvider, I18nProvider } from "../i18n";
import { NotificationContextProvider } from "../notification";
import { AppRouter } from "../routing";
import { DefaultTitleContextProvider } from "../title";
import { ErrorBoundary } from "../errorBoundary";
import { DataProvider, DataProviderContext } from "../data";
import { QueryClient, QueryClientProvider } from "@tanstack/react-query";

export interface CoreAppContextProps extends CoreAppRoutesProps {
  /**
   * The authentication provider for security and permissions
   *
   * @see https://marmelab.com/react-admin/Authentication.html
   * @example
   * import authProvider from './authProvider';
   *
   * const App = () => (
   *     <Admin authProvider={authProvider}>
   *         ...
   *     </Admin>
   * );
   */
  authProvider?: AuthProvider;

  /**
   * The content displayed when the user visits the /auth-callback page, used for redirection by third-party authentication providers
   *
   * @see https://marmelab.com/react-admin/Admin.html#authcallbackpage
   * @example
   * import { Admin } from 'react-admin';
   * import { dataProvider } from './dataProvider';
   * import { authProvider } from './authProvider';
   * import MyAuthCallbackPage from './MyAuthCallbackPage';
   *
   * const App = () => (
   *     <Admin
   *         authCallbackPage={MyAuthCallbackPage}
   *         authProvider={authProvider}
   *         dataProvider={dataProvider}
   *     >
   *         ...
   *     </Admin>
   * );
   */
  authCallbackPage?: ComponentType<any> | null;

  /**
   * The base path for all URLs generated by react-admin.
   *
   * @see https://marmelab.com/react-admin/Admin.html#using-react-admin-in-a-sub-path
   * @example
   * import { Admin } from 'react-admin';
   * import { BrowserRouter } from 'react-router-dom';
   * import { dataProvider } from './dataProvider';
   *
   * const App = () => (
   *     <BrowserRouter>
   *         <Admin basename="/admin" dataProvider={dataProvider}>
   *              ...
   *         </Admin>
   *    </BrowserRouter>
   * );
   */
  basename?: string;

  dataProvider?: DataProvider;

  /**
   * The component displayed when an error is caught in a child component
   * @see https://marmelab.com/react-admin/Admin.html#error
   * @example
   * import { Admin } from 'react-admin';
   * import { MyError } from './error';
   *
   * const App = () => (
   *     <Admin error={MyError}>
   *         ...
   *     </Admin>
   * );
   */
  error?: ComponentType<any> | ReactElement | null;

  /**
   * The internationalization provider for translations
   *
   * @see https://marmelab.com/react-admin/Translation.html
   * @example
   * // in src/i18nProvider.js
   * import polyglotI18nProvider from 'ra-i18n-polyglot';
   * import fr from 'ra-language-french';
   *
   * export const i18nProvider = polyglotI18nProvider(() => fr, 'fr');
   *
   * // in src/App.js
   * import { Admin } from 'react-admin';
   * import { dataProvider } from './dataProvider';
   * import { i18nProvider } from './i18nProvider';
   *
   * const App = () => (
   *     <Admin dataProvider={dataProvider} i18nProvider={i18nProvider}>
   *         ...
   *     </Admin>
   * );
   */
  i18nProvider?: I18nProvider;

  /**
   * The component displayed when the user visits the /login page
   * @see https://marmelab.com/react-admin/Admin.html#loginpage
   * @example
   * import { Admin } from 'react-admin';
   * import { dataProvider } from './dataProvider';
   * import { authProvider } from './authProvider';
   * import MyLoginPage from './MyLoginPage';
   *
   * const App = () => (
   *     <Admin
   *         loginPage={MyLoginPage}
   *         authProvider={authProvider}
   *         dataProvider={dataProvider}
   *     >
   *         ...
   *     </Admin>
   * );
   */
  loginPage?: ComponentType<any> | null;

  /**
   * The react-query client
   *
   * @see https://marmelab.com/react-admin/Admin.html#queryclient
   * @example
   * import { Admin } from 'react-admin';
   * import { QueryClient } from '@tanstack/react-query';
   *
   * const queryClient = new QueryClient({
   *     defaultOptions: {
   *         queries: {
   *             retry: false,
   *             structuralSharing: false,
   *         },
   *         mutations: {
   *             retryDelay: 10000,
   *         },
   *     },
   * });
   *
   * const App = () => (
   *     <Admin queryClient={queryClient} dataProvider={...}>
   *         ...
   *     </Admin>
   * );
   */
  queryClient?: QueryClient;

  /**
   * The adapter for storing user preferences
   *
   * @see https://marmelab.com/react-admin/Admin.html#store
   * @example
   * import { Admin, memoryStore } from 'react-admin';
   *
   * const App = () => (
   *     <Admin dataProvider={dataProvider} store={memoryStore()}>
   *         ...
   *     </Admin>
   * );
   */
  store?: Store;

  /**
   * The title of the error page
   * @see https://marmelab.com/react-admin/Admin.html#title
   * @example
   * import { Admin } from 'react-admin';
   * import { dataProvider } from './dataProvider';
   *
   * const App = () => (
   *    <Admin title="My Admin" dataProvider={dataProvider}>
   *       ...
   *   </Admin>
   * );
   */
  title?: string | ReactElement | null;
}

export function CoreAppContext(props: CoreAppContextProps) {
  const {
    authProvider,
    authCallbackPage: LoginCallbackPage,
    basename,
    catchAll,
    children,
    dashboard,
    dataProvider,
    error,
    i18nProvider,
    initialLocation,
    layout,
    loading,
    loginPage: LoginPage,
    queryClient,
    ready,
    requireAuth,
    store = defaultStore,
    title,
  } = props;

  const finalQueryClient = useMemo(
    () => queryClient || new QueryClient(),
    [queryClient]
  );

  return (
    <AuthContext.Provider value={authProvider}>
      <DataProviderContext.Provider value={dataProvider}>
        <I18nContextProvider value={i18nProvider}>
          <NotificationContextProvider>
            <StoreContextProvider value={store}>
              <QueryClientProvider client={finalQueryClient}>
                <AppRouter basename={basename}>
                  <DefaultTitleContextProvider value={title}>
                    <ErrorBoundary error={error}>
                      <Routes>
                        {LoginPage != null ? (
                          <ReactRoute
                            path="/login"
                            element={<LoginPage />}
                          />
                        ) : null}

                        {LoginCallbackPage != null ? (
                          <ReactRoute
                            path="/auth-callback"
                            element={<LoginCallbackPage />}
                          />
                        ) : null}

                        <ReactRoute
                          path="/*"
                          element={
                            <CoreAppRoutes
                              catchAll={catchAll}
                              dashboard={dashboard}
                              initialLocation={initialLocation}
                              layout={layout}
                              loading={loading}
                              requireAuth={requireAuth}
                              ready={ready}
                            >
                              {children}
                            </CoreAppRoutes>
                          }
                        />
                      </Routes>
                    </ErrorBoundary>
                  </DefaultTitleContextProvider>
                </AppRouter>
              </QueryClientProvider>
            </StoreContextProvider>
          </NotificationContextProvider>
        </I18nContextProvider>
      </DataProviderContext.Provider>
    </AuthContext.Provider>
  );
}
