import {
  ComponentType,
  ReactElement,
  ReactNode,
  useEffect,
  useState
} from 'react';
import { Navigate, Route, Routes } from 'react-router-dom';
import { LogoutOnMount, useCheckAuth } from '../auth';
import { InitialLocationContextProvider } from '../routing';
import { getReactElement } from '../util';
import { DefaultLayout } from './DefaultLayout';
import { CoreLayoutProps } from './types';
import { useConfigureRoutesFromChildren } from './useConfigureRoutesFromChildren';
import { useScrollToTop } from '../scrollPosition';
import { HasDashboardContextProvider } from './HasDashboardContextProvider';

export interface CoreAppRoutesProps {
  /**
   * A catch-all react component to display when the URL does not match any
   *
   * @see https://marmelab.com/react-admin/Admin.html#catchall
   * @example
   * // in src/NotFound.js
   * import Card from '@mui/material/Card';
   * import CardContent from '@mui/material/CardContent';
   * import { Title } from 'react-admin';
   *
   * export const NotFound = () => (
   *     <Card>
   *         <Title title="Not Found" />
   *         <CardContent>
   *             <h1>404: Page not found</h1>
   *         </CardContent>
   *     </Card>
   * );
   *
   * // in src/App.js
   * import { Admin } from 'react-admin';
   * import { dataProvider } from './dataProvider';
   * import { NotFound } from './NotFound';
   *
   * const App = () => (
   *     <Admin catchAll={NotFound} dataProvider={dataProvider}>
   *         ...
   *     </Admin>
   * );
   */
  catchAll?: ComponentType<any> | ReactElement | null;

  children?: ReactNode;

  /**
   * @example
   * import { WithPermissions } from "rwas";
   * import HomepageView from "./views/Homepage";
   *
   * const authParams = {
   *     params: { route: 'homepage' },
   * };
   *
   * const firstpage = (
   *     <WithPermissions
   *         authParams={authParams}
   *         component={HomepageView}
   *     />
   * )
   */
  dashboard?: ComponentType | ReactElement | null;

  /**
   * @example
   * import { WithPermissions } from "rwas";
   * import HomepageView from "./views/Homepage";
   *
   * const authParams = {
   *     params: { route: 'homepage' },
   * };
   *
   * const firstpage = (
   *     <WithPermissions
   *         authParams={authParams}
   *         component={HomepageView}
   *     />
   * )
   */
  homepage?: ComponentType<any> | ReactElement | null;

  initialLocation?: string;

  /**
   * The main app layout component
   *
   * @see https://marmelab.com/react-admin/Admin.html#layout
   * @example
   * import { Admin, Layout } from 'react-admin';
   *
   * const MyLayout = ({ children }) => (
   *     <Layout appBarAlwaysOn>
   *        {children}
   *     </Layout>
   * );
   *
   * export const App = () => (
   *     <Admin dataProvider={dataProvider} layout={MyLayout}>
   *         ...
   *     </Admin>
   * );
   */
  layout?: ComponentType<CoreLayoutProps>;

  /**
   * The component displayed while fetching the auth provider if the admin child is an async function
   */
  loading?: ComponentType<any> | ReactElement | null;

  /**
   * The page to display when the admin has no Resource children
   *
   * @see https://marmelab.com/react-admin/Admin.html#ready
   * @example
   * import { Admin } from 'react-admin';
   *
   * const Ready = () => (
   *     <div>
   *         <h1>Admin ready</h1>
   *         <p>You can now add resources</p>
   *     </div>
   * )
   *
   * const App = () => (
   *     <Admin ready={Ready}>
   *         ...
   *     </Admin>
   * );
   */
  ready?: ComponentType<any> | ReactElement | null;

  /**
   * Flag to require authentication for all routes. Defaults to false.
   *
   * @see https://marmelab.com/react-admin/Admin.html#requireauth
   * @example
   * import { Admin } from 'react-admin';
   * import { dataProvider } from './dataProvider';
   * import { authProvider } from './authProvider';
   *
   * const App = () => (
   *     <Admin
   *         requireAuth
   *         authProvider={authProvider}
   *         dataProvider={dataProvider}
   *     >
   *         ...
   *     </Admin>
   * );
   */
  requireAuth?: boolean;
}

export function CoreAppRoutes(props: CoreAppRoutesProps) {
  useScrollToTop();

  const [routes, status] = useConfigureRoutesFromChildren(props.children);

  const {
    catchAll: catchAllElement,
    dashboard,
    homepage: homepageElement,
    initialLocation,
    layout: Layout = DefaultLayout,
    loading: loadingElement,
    requireAuth,
    ready: readyElement,
  } = props;

  const [onlyAnonymousRoutes, setOnlyAnonymousRoutes] = useState(requireAuth);
  const [checkAuthLoading, setCheckAuthLoading] = useState(requireAuth);
  const checkAuth = useCheckAuth();

  useEffect(() => {
    if (requireAuth) {
      // do not log the user out on failure to allow access to custom routes with no layout
      // for other routes, the LogoutOnMount component will log the user out
      checkAuth(undefined, false)
        .then(() => {
          setOnlyAnonymousRoutes(false);
        })
        .catch(() => { })
        .finally(() => {
          setCheckAuthLoading(false);
        });
    }
  }, [checkAuth, requireAuth]);

  if (status === 'empty') {
    if (!readyElement) {
      throw new Error(
        'The admin is empty. Please provide an empty component, ' +
        'or pass Route or CustomRoutes as children.'
      );
    }
    return getReactElement(readyElement);
  }

  if (status === 'loading' || checkAuthLoading) {
    return (
      <Routes>
        <Route
          path="*"
          element={
            <div style={{ height: '100vh' }}>
              {loadingElement ? getReactElement(loadingElement) : 'loading...'}
            </div>
          }
        />
      </Routes>
    );
  }

  if (onlyAnonymousRoutes) {
    return (
      <Routes>
        <Route
          path="*"
          element={<LogoutOnMount />}
        />
      </Routes>
    );
  }

  return (
    <InitialLocationContextProvider
      location={initialLocation}
      target="app"
    >
      <Routes>
        <Route
          path="/*"
          element={
            <HasDashboardContextProvider value={!!dashboard}>
              <Layout>
                <Routes>
                  {routes}

                  <Route
                    path="/"
                    element={
                      homepageElement
                        ? (getReactElement(homepageElement))
                        : initialLocation
                          ? (<Navigate to={initialLocation} />)
                          : null
                    }
                  />

                  {dashboard
                    ? (<Route path="/" element={getReactElement(dashboard)} />)
                    : (initialLocation && initialLocation !== "/")
                      ? (<Route path="/" element={<Navigate to={initialLocation} />} />)
                      : null}

                  {catchAllElement ? (
                    <Route
                      path="*"
                      element={getReactElement(catchAllElement)}
                    />
                  ) : null}
                </Routes>
              </Layout>
            </HasDashboardContextProvider>
          }
        />
      </Routes>
    </InitialLocationContextProvider>
  );
}
